#!/usr/local/bin/perl
#
# NAME:
#	CAindex.pl - manipulates CA index
#
# SYNOPSIS:
#	CAindex.pl [options] "index"
#
# DESCRIPTION:
#	This script runs reads the CA "index" file, and prints to
#	stdout new records which reflect expired and revoked
#	certificates.
#
#	Options:
#
#	-e "re"
#		Select certificates that match "re".
#
#	-s "re"
#		Select the certificates with serial numbers that match
#		"re".
#		
#	-r	Revoke selected certificates.
#
#	-u	Output user:cert suitable for inclusion in an
#		ssl.users file.  This is useful for keeping an 
#		up to date ssl.global that can be distributed via
#		SSLrdist(8) to all hosts.
#
#	-U	Output cert:passwd suitable for inclusion in an ApacheSSL
#		server's passwd file.  
#
#	-E "re"	When producing ssl.users output, if the email address
#		matches "re", add the user part as a valid target for
#		the certificate.
#
# SEE ALSO:
#	SSLeay FAQ.
#
# AUTHOR:
#	Simon J. Gerraty <sjg@quick.com.au>
#

# RCSid:
#	$Id: CAindex.pl,v 1.7 1999/02/17 07:30:52 sjg Exp $
#
#	@(#) Copyright (c) 1996 Simon J. Gerraty
#
#	This file is provided in the hope that it will
#	be of use.  There is absolutely NO WARRANTY.
#	Permission to copy, redistribute or otherwise
#	use this file is hereby granted provided that 
#	the above copyright notice and this notice are
#	left intact. 
#      
#	Please send copies of changes and bug-fixes to:
#	sjg@quick.com.au
#

require 'mktime.pl';
require 'strftime.pl';
require 'getopts.pl';

$Today = time;
$opt_E = $opt_e = '^#';		# won't happen
$opt_w = 30;			# warn if expires in less than 30 days

&Getopts('T:vdrs:e:w:uUE:S:');

if ($opt_U ne '') {
  $opt_u = $opt_U;
}
if ($opt_T ne '') {
  $opt_T = &mktime($opt_T);
} else {
  $opt_T = $Today;
}
if ($opt_u ne '' && $opt_e eq '^#') {
  $opt_e = '.';
}

#
# Try not to create a security problem - eg a cert that will accidentally
# allow logons as a system account
#
@SysAccounts = ('root', 'uucp', 'bin', 'smtp', 'daemon', 'sys', 'adm',
		'rook', 'toor', 'nobody', 'operator', 'new', 'ftp', 'games');
if ($opt_S ne '') {
  $opt_S =~ y/A-Z/a-z/;
  push(@SysAccounts, split(/,/, $opt_S));
}
# finally get any local accounts with uid < 100
if (open(I, "< /etc/passwd")) {
  while (<I>) {
    y/A-Z/a-z/;
    @f = split(/:/);
    push(@SysAccounts, $f[0]) if ($f[2] < 100);
  }
  close I;
}

foreach $k (@SysAccounts) {
  $SysAccounts{$k} = 1;
}

		
&main;
exit 0;

sub main {
  while (<>) {
    chop;

    # ($state,$notAfter,$revoked,$serial,$unknown,$DN)
    ($state,@data) = split(/\t/);
    
    if ($state eq 'V' || $state eq 'E') {
      $date = $data[0];
      if ($date !~ m/^9/) {
	$date = "20$date";
      }
      
      $expires = &mktime($date);
      $days = int(($expires - $opt_T) / 86400);

      if ($days <= 0) {
	if ($opt_v ne '' && $state eq 'V') {
	  printf STDERR "Expired %s:\n\t$_\n", &strftime('%C', $expires);
	}
	$state = 'E';
      } else {
	$state = 'V';
	if ($opt_u eq '' && $opt_r ne '') {
	  if (($opt_s ne '' && $data[2] =~ m/^$opt_s$/i) || m/$opt_e/i) {
	    $data[1] = &strftime('%y%m%d%H%M%SZ', $Today);
	    $state = 'R';
	  }
	}
	if ($state eq 'V') {
	  if ($opt_u ne '') {
	    if (m/$opt_e/i &&
		$data[$#data] =~ m,/CN=([^./\-]+)([^/]*)/.*Email=([^\@]+),i) {
	      $name=$1;
	      $domain=$2;
	      $email=$3;
	      $email =~ y/A-Z/a-z/;
	      next if ($domain =~ m/^\./ && $opt_E !~ m/root/);
	      if ($opt_U ne '') {
		# We want input for apache's SSLFakeBasicAuth
		# the passwd (its a _long_ story...) is magic
		$u = "$data[$#data]:xxj31ZMTZzkVA\n";
	      } else {
		if ($email ne 'certs') {
		  if ($email =~ m/^$opt_E$/i && $email ne $name && $SysAccounts{$email} eq '') {
		    $u = "$name,$email:$data[$#data]\n";
		  } else {
		    $u = "$name:$data[$#data]\n";
		  }
		}
		# This is necessary so that both SSLeay 0.6 and 0.8 work
		foreach $st ('SP', 'ST') {
		  $u =~ s,/S[PT]=,/$st=,;
		  print $u;
		}
	      }
	    }
	  } elsif ($opt_d ne '') {
	    printf STDERR "%d:$state:%s:%s\n", $days, $data[2], $data[$#data];
	  } elsif ($days < $opt_w) {
	    printf STDERR "Warning: $data[$#data] expires in %d days\n", $days;
	  }
	}
      }
    }
    next if ($opt_u ne '');
    print join("\t",($state,@data)),"\n";
  }
}


  
